/**
*	@file imagereader.h
*	@class ImageReader
*	@author COS121 Class
*	@version 1.0.0.0 beta
*
*	@brief A class for reading an image file
*
*	@section Description
* 	This class provides the functionality of opening a file, reading individual indexes of the buffer and closing the file again.
*
*	@section License
*	Copyright belongs to COS121
*
*/

#ifndef IMAGEREADER_H
#define IMAGEREADER_H

/**
	@def MAX_FILE_SIZE
	Holds the maximum size an image may have.
*/

#define MAX_FILE_SIZE 4096

#include "imagefile.h"

class ImageReader
{

	public:

		/**
		*	A constructor for creating an image reader.
		*/
		ImageReader();
	
		/**
		*	A destructor for deleting an image reader.
		*/
		~ImageReader();
	
		/**
		*	This function opens a file.
		*	@param filepath The path to the file
		*	@param action The action used to open the file. ("write", "read", "readwrite").
		*	@return A void pointer to the allocated buffer. Each index holds an array of values. Each index represents a band.
		*/
		virtual void** open(string filepath, string action);
	
		/**
		*	This function is for accessing (read/write) values at a specific index in the buffer.
		*	@param buffer Holds the buffer with the image.
		*	@param value The pointer to the value used to store the read value or the value to be written.
		*	@param index The index in the buffer to read or write.
		*	@param action The action to execute. ('w' for write' and 'r' for read').
		*	@param band The band to be accessed.
		*/
		virtual void access(void** buffer, double *value, int index, char action, int band);
	
		/**
		*	This function closes a file.
		*	@param force True if the file should be closed with force.
		*	@return True if the close was successful, otherwise false.
		*	@todo Add extra functionality here!
		*/
		virtual bool close(bool force = 0);
	
	private:
		
		/**
			@brief The pointer to the opened file
		*/
		ImageFile *file;

};

#endif

